//--------------------------------------------------------------------------------------------------------------
//  １文字・部分一致検索プラグイン
//    検索キー一括作成ボタン
//  By okiyasu
//--------------------------------------------------------------------------------------------------------------
jQuery.noConflict();

(function($, PLUGIN_ID) {
  "use strict";

  // 文字列のフィールドコードを取得
  var config = kintone.plugin.app.getConfig("pplmlhhkfkeepnechnlbjplhogpmcpjf");

  var storage = sessionStorage;
  var appId = kintone.app.getId();

  function showMessage(message, title, type) {
    if (message === 'undefined' || message === '') {
      return;
    }
    title = title || '';
    type = type || 'success'; //"warning", "error", "success" and "info"

    swal({
        title: title,
        text: message,
        type: type,
        confirmButtonText: "OK",
        closeOnConfirm: true
      },
      function() {
        //なにもしない
        ;
      });
  }

  function checkField() {
    var field_list = ['plugin-search-field', 'plugin-key-field'];
    if (!config)
      return false;
    for (var i = 0; i < field_list.length; i++) {
      if (config[field_list[i]] === undefined || config[field_list[i]] === '' ||
        config[field_list[i]] === null) {
        return false;
      }
    }
    return true;
  }

  function fetchRecords(appId, opt_offset, opt_limit, opt_records) {
    var offset = opt_offset || 0;
    var limit = opt_limit || 100;
    var allRecords = opt_records || [];
    var params = {
      app: appId,
      query: config['plugin-key-field'] + ' = "" order by $id asc limit ' + limit + ' offset ' + offset
    };
    return kintone.api(kintone.api.url('/k/v1/records', true), 'GET', params).then(function(resp_get) {
      var body = createPutData(resp_get['records']);
      return kintone.api(kintone.api.url('/k/v1/records', true), 'PUT', body).then(function(resp_put) {
        console.log(resp_put);
      }, function(error) {
        console.log(error);
      }).then(function() {
        if (resp_get.records.length === limit) {
          return fetchRecords(appId);
        }
        return allRecords;
      });
    }, function(error) {
      console.log(error);
    });
  }

  function createPutData(records) {
    var body = {
      "app": kintone.app.getId(),
      "records": []
    };

    records.forEach(function(record, k) {
      var keycode = config['plugin-key-field'];
      var keydata = '';
      if (record[config['plugin-search-field']].value) {
        var key = record[config['plugin-search-field']].value;
        var words = [];
        for (var i = 1; i <= key.length; i++) {
          for (var j = 0; j < key.length; j++) {
            if (i + j > key.length)
              continue;
            if (words.indexOf(key.slice(j, j + i)) >= 0)
              continue;
            words.push(key.slice(j, j + i));
          }
        }
        keydata = words.join(' ');
      } else {
        keydata = '';
      }

      var data = {
        "id": record["$id"]["value"],
        "record": {}
      };
      data['record'][keycode] = {
        "value": keydata
      };
      body["records"].push(data);
    });

    return body;
  }

  function createKey() {
    //configに値が無いか、フィールドの設定が足りなければ何もせず終了
    if (!checkField()) {
      showMessage('エラーが発生しました。プラグイン設定内容をご確認ください', 'エラー', 'error');
      return e;
    }

    showSpinner(); // スピナー表示
    fetchRecords(kintone.app.getId()).then(function(records) {
      hideSpinner(); // スピナー非表示
      location.reload(true);
      console.log(records.length);
    });
  }

  // スピナーを動作させる関数
  function showSpinner() {
    // 要素作成等初期化処理
    if ($('.kintone-spinner').length == 0) {
      // スピナー設置用要素と背景要素の作成
      var spin_div = $('<div id ="kintone-spin" class="kintone-spinner"></div>');
      var spin_bg_div = $('<div id ="kintone-spin-bg" class="kintone-spinner"></div>');

      // スピナー用要素をbodyにappend
      $(document.body).append(spin_div, spin_bg_div);

      // スピナー動作に伴うスタイル設定
      $(spin_div).css({
        'position': 'fixed',
        'top': '50%',
        'left': '50%',
        'z-index': '510',
        'background-color': '#fff',
        'padding': '26px',
        '-moz-border-radius': '4px',
        '-webkit-border-radius': '4px',
        'border-radius': '4px'
      });
      $(spin_bg_div).css({
        'position': 'absolute',
        'top': '0px',
        'left': '0px',
        'z-index': '500',
        'width': '100%',
        'height': '200%',
        'background-color': '#000',
        'opacity': '0.5',
        'filter': 'alpha(opacity=50)',
        '-ms-filter': "alpha(opacity=50)"
      });

      // スピナーに対するオプション設定
      var opts = {
        'color': '#000'
      };

      // スピナーを作動
      new Spinner(opts).spin(document.getElementById('kintone-spin'));
    }

    // スピナー始動（表示）
    $('.kintone-spinner').show();
  }

  // スピナーを停止させる関数
  function hideSpinner() {
    // スピナー停止（非表示）
    $('.kintone-spinner').hide();
  }

  kintone.events.on('app.record.index.show', function(event) {
    var button_name = 'ohks_advanced_search_keycreate_button';
    // 増殖バグを防ぐ
    if (document.getElementById(button_name) !== null) {
        return;
    }

    var keycreatebutton = document.createElement('button');
    keycreatebutton.id = button_name;
    keycreatebutton.innerHTML = '検索キー作成';
    keycreatebutton.classList.add('kintoneplugin-button-normal');
    kintone.app.getHeaderMenuSpaceElement().appendChild(keycreatebutton);
    // ボタンクリック時の処理
    keycreatebutton.onclick = function() {
      createKey();
    }
    return event;
  });
})(jQuery, kintone.$PLUGIN_ID);
